from django import forms
from django.contrib.auth.models import User
from models import Room
import models

import re

class RegistrationForm(forms.Form):
	username = forms.CharField(label=u'Username', max_length=30)
	email = forms.EmailField(label=u'Email')
	password1 = forms.CharField(label=u'Password', widget=forms.PasswordInput())
	password2 = forms.CharField(label=u'Password (Again)', widget=forms.PasswordInput())

	def clean_password2(self):
		if 'password1' in self.cleaned_data:
			password1 = self.cleaned_data['password1']
			password2 = self.cleaned_data['password2']
			if password1 == password2:
				return password2
			raise forms.ValidationError('Passwords do not match.')

	def clean_username(self):
		username = self.cleaned_data['username']
		if not re.search(r'^\w+$', username):
			raise forms.ValidationError('Username can only contain '
					'alphanumeric characters and the underscore.')
		try:
			User.objects.get(username=username)
		except User.DoesNotExist:
			return username
		raise forms.ValidationError('Username is already taken.')

class ChatRoomForm(forms.Form):
	roomName = forms.CharField(label=u'roomName', max_length=255)
	comment = forms.CharField(label=u'Comment', max_length=255)	
	password1 = forms.CharField(label=u'password', widget=forms.PasswordInput())
	password2 = forms.CharField(label=u'password (Again)', widget=forms.PasswordInput())
	public = forms.BooleanField(label=u'public', widget=forms.CheckboxInput(), required=False)

	def clean_roomName(self):
		roomName = self.cleaned_data['roomName']
		if not re.search(r'^\w+$', roomName):
			raise forms.ValidationError('Username can only contain '
					'alphanumeric characters and the underscore.')
		try:
			Room.objects.get(roomName=roomName)
		except Room.DoesNotExist:
			return roomName
		raise forms.ValidationError('roomName is already taken.')
	
	def clean_password2(self):
		if 'password1' in self.cleaned_data:
			password1 = self.cleaned_data['password1']
			password2 = self.cleaned_data['password2']
			if password1 == password2:
				return password2
			raise forms.ValidationError('Passwords do not match.')	
	
class JoinRoomForm(forms.Form):
	roomName = forms.CharField(max_length=255)
	password = forms.CharField(max_length=255, required=False)
	
	def clean_password(self):
		roomName=self.cleaned_data['roomName']
		password=self.cleaned_data['password']
		r = Room.objects.get_for_object(roomName)
		if r.public:
			return True
		else:
			if r.password == password:
				return True
		raise forms.ValidationError('Incorrect Password.') 
		#return False
		
class PollForm(forms.ModelForm):
	class Meta:
		model = models.Poll
		
class ChoiceForm(forms.Form):
	choice = forms.CharField(max_length=255)
	
	def __init__(self, poll=None, *args, **kwargs):
	        self.poll = poll
        	super(ChoiceForm, self).__init__(*args, **kwargs)

	def save(self):
	        choice = models.Choice(poll = self.poll, choice = self.cleaned_data['choice'])
	        choice.save()
		
