from django.http import HttpResponse, HttpResponseRedirect, Http404
from django.shortcuts import render_to_response
from django.contrib.auth import authenticate, login
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.contrib.auth import logout
from django.views.decorators.csrf import csrf_protect
from django.template import RequestContext

from models import *
from forms import *

from datetime import datetime
import time
import hashlib

@csrf_protect
def home(request):
	wrong_PW = False
	if request.method == 'POST':
		form = JoinRoomForm(request.POST)
		if form.is_valid():
			roomName = form.cleaned_data['roomName']
			pw = form.cleaned_data['password']
			r = Room.objects.get_for_object(roomName)
			if r.public:
				append = 'public'
			else:
				hashf = hashlib.sha1()
				hashf.update(r.password)
				result = hashf.hexdigest()
				append = str(result) 
			return HttpResponseRedirect('rooms/'+roomName+'/'+append+'/')
		else:
			wrong_PW = True			
	rooms = Room.objects.order_by('+roomName')	
	variables = RequestContext(request, {
		'rooms': rooms,
		'wrong_PW': wrong_PW,	
		})
	return render_to_response('home.html', variables)

@csrf_protect
def signup(request):
	if request.method == 'POST':
		form = RegistrationForm(request.POST)
		if form.is_valid():
			user = User.objects.create_user(
					username = form.cleaned_data['username'],
					password = form.cleaned_data['password1'],
					email = form.cleaned_data['email']
			)			
			return HttpResponseRedirect('success/')
	else:
		form = RegistrationForm()
	variables = RequestContext(request, {
		'form': form
		})
	return render_to_response(
			'registration/signup.html',
			variables
		)

@login_required
@csrf_protect
def newroom(request):
	if request.method == 'POST':
		form = ChatRoomForm(request.POST)
		if form.is_valid():
			room = Room.objects.create(
					request.user, 
					form.cleaned_data['roomName'],
					form.cleaned_data['comment'],
					form.cleaned_data['password1'],
					form.cleaned_data['public']
			)
			room.save()
			return HttpResponseRedirect('success/')
	else:
		form = ChatRoomForm()
	variables = RequestContext(request, {
		'form':form		
		})
	return render_to_response(
			'roomcreate.html',
			variables	
		)

@login_required
@csrf_protect
def rooms(request, roomName=None, append=None):
	rname = roomName
	append = append
	isowner = False
	#ismember = False
	r = Room.objects.get_for_object(rname)
	u = request.user
	if r.owner==u:
		isowner = True
	#members = r.members.all()
	if r.public:
		pass
	else:
		hashf = hashlib.sha1()
		hashf.update(r.password)
		result = hashf.hexdigest()
		appendnew = str(result)
		if appendnew == append:
			pass
		else:
			return HttpResponseRedirect('failtoenter/')
	ru = RoomUser.objects.all().filter(rname=r.roomName, uname=request.user.username)
	if len(ru)==0:
		ru_new = RoomUser(rname=r.roomName, uname=u.username, talk_permission=True)
		ru_new.inroom = True
		ru_new.save()
	else:
		for i in ru:
			i.inroom = True
			i.save()
    	return render_to_response('rooms.html', {'chat_id':r.pk}, context_instance=RequestContext(request, {'room':r, 'isowner':isowner}))

@login_required
def send(request):
    '''
    Expects the following POST parameters:
    chat_room_id
    message
    '''
    p = request.POST
    r = Room.objects.get(id=int(p['chat_room_id']))
    entries = RoomUser.objects.all().filter(rname=r.roomName, uname=request.user.username)
    for i in entries:
    	if i.talk_permission==True:
    		r.say(request.user, p['message'])
    		break

    return HttpResponse('')

@login_required
def sync(request):
   
    if request.method != 'POST':
        raise Http404
    post = request.POST

    if not post.get('id', None):
        raise Http404

    r = Room.objects.get(id=post['id'])
    
    lmid = r.last_message_id()
    lmtm = float(time.mktime(r.last_message_time().timetuple()))    
    
    return HttpResponse(jsonify({'last_message_id':lmid, 'last_message_time':lmtm}))

@login_required
def receive(request):

    if request.method != 'POST':
        raise Http404
    post = request.POST

    if not post.get('id', None) or not post.get('offset', None):
        raise Http404
    
    try:
        room_id = int(post['id'])
    except:
        raise Http404

    try:
        offset = float(post['offset'])+1
    except:
        offset = float(time.mktime(datetime.now().timetuple()))
    
    r = Room.objects.get(id=room_id)
    date_offset = datetime.fromtimestamp(offset)

    m = r.messages(after_date=date_offset)
   
    return HttpResponse(jsonify(m, ['id','author','message','type']))


@login_required
def join(request):
    '''
    Expects the following POST parameters:
    chat_room_id
    message
    '''
    p = request.POST
    r = Room.objects.get(id=int(p['chat_room_id']))
    r.join(request.user)
    return HttpResponse('')


@login_required
def leave(request):
    '''
    Expects the following POST parameters:
    chat_room_id
    message
    '''
    p = request.POST
    r = Room.objects.get(id=int(p['chat_room_id']))
    r.leave(request.user)
    ru = RoomUser.objects.all().filter(rname=r.roomName, uname=request.user.username)
    for i in ru:
	i.inroom = False
	i.save()
    if r.deleted:
    	ru = RoomUser.objects.all().filter(rname=r.roomName, inroom=True)
    	if len(ru)==0:
    		msgs = Message.objects.filter(room=r)
    		for m in msgs:
    			m.delete()
    		ru = RoomUser.objects.filter(rname=r.roomName)
    		for i in ru:
    			i.delete()
    		r.delete()
    		
    return HttpResponse('')
    
@login_required
def setroom(request):
	p = request.POST
	r = Room.objects.get(id=int(p['id']))
	rstate = p['rstate']
	if(rstate=='public'):
		r.public = True
	else:
		r.public = False
	r.save()
	return HttpResponse('')

@login_required
def refreshlist(request):
	p = request.POST
	r = Room.objects.get(id=int(p['id']))
	entries = RoomUser.objects.all().filter(rname=r.roomName, inroom=True)
	try:
        	import json
	except:
	        import django.utils.simplejson as json
	        
	out = []
	for i in entries:
		tmp = {}
		tmp['uname']=i.uname
		if i.talk_permission:
			tmp['permission']='Enabled'
		else:
			tmp['permission']='Disabled'
		out.append(tmp)
	return HttpResponse(json.dumps(out))
	
@login_required
def disableall(request):
	p = request.POST
	r = Room.objects.get(id=int(p['chat_room_id']))
	entries = RoomUser.objects.all().filter(rname=r.roomName)
	for i in entries:
		i.talk_permission=False
		i.save()
	return HttpResponse('')
	
@login_required
def enableall(request):
	p = request.POST
	r = Room.objects.get(id=int(p['chat_room_id']))
	entries = RoomUser.objects.all().filter(rname=r.roomName)
	for i in entries:
		i.talk_permission=True
		i.save()
	return HttpResponse('')
	
@login_required
def disable(request):
	p = request.POST
	r = Room.objects.get(id=int(p['chat_room_id']))
	uname = p['uname']
	entries = RoomUser.objects.all().filter(rname=r.roomName, uname=uname)
	for i in entries:
		i.talk_permission=False
		i.save()
	return HttpResponse('')

@login_required
def enable(request):
	p = request.POST
	r = Room.objects.get(id=int(p['chat_room_id']))
	uname = p['uname']
	entries = RoomUser.objects.all().filter(rname=r.roomName, uname=uname)
	for i in entries:
		i.talk_permission=True
		i.save()
	return HttpResponse('')

@login_required
def commentedit(request):
	p = request.POST
	r = Room.objects.get(id=int(p['chat_room_id']))
	r.comment = p['comment']
	r.save()
	return HttpResponse('')
	
@login_required
def changepw(request):
	p = request.POST
	r = Room.objects.get(id=int(p['chat_room_id']))
	tmp = 'a'
	if r.password == p['oldpw']:
		if p['newpw'] == p['newpw2']:
			r.password = p['newpw']
			r.save()
			tmp = 'b'
		else:
			tmp = 'c'
	return HttpResponse(jsonify({'result':tmp}))
		
@login_required
def deleteroom(request):
	p = request.POST
	r = Room.objects.get(id=int(p['chat_room_id']))
	tmp = False
	if r.password == p['password']:
		r.deleted = True
		r.save();
		tmp = True
	return HttpResponse(jsonify({'deleted':tmp}))

def jsonify(object, fields=None, to_dict=False):
    '''Simple convert model to json'''
    try:
        import json
    except:
        import django.utils.simplejson as json
 
    out = []
    if type(object) not in [dict,list,tuple] :
        for i in object:
            tmp = {}
            if fields:
                for field in fields:
                    tmp[field] = unicode(i.__getattribute__(field))
		tmp['time'] = float(time.mktime(i.timestamp.timetuple()));
            else:
                for attr, value in i.__dict__.iteritems():
                    tmp[attr] = value
            out.append(tmp)
    else:
        out = object
    if to_dict:
        return out
    else:
        return json.dumps(out)

